/***************************************************************************
 *   Copyright (C) 2007 by    *
 *      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "acstoroid.h"
#include <qtimer.h>
#include <qsettings.h>

	char tmBuffer[80];
/*
  timestamp() 
  send a timestamp to standard output
*/
void timestamp() 
{
	struct tm *mytime;
	time_t myTime_t;
	myTime_t = time(NULL);
	mytime = localtime(&myTime_t);
	strftime(tmBuffer, 79, "%Y-%m-%d %H:%M:%S ", mytime);
	return;
}

acstoroid::acstoroid(QWidget *parent) : QWidget(parent)
{
	// setup User Interface
	try {
		ui.setupUi(this);
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "cannot setup UI" << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception, cannot setup UI" << endl;
	}
	// connect to control access server
	try {
                // plc_server = new Tango::DeviceProxy("ken/plcaccess/0");
                // plc_server = new Tango::DeviceProxy("booster/accessi/plc_accessi");
		// plc_server = new Tango::DeviceProxy("ken/plc/fsafety");
		plc_server = new Tango::DeviceProxy("f/access_control/safety");
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "Fatal Error: Access Control Device Server not responding, the application will be terminated immediately" << endl;
		exit(1);
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "Fatal Error: Access Control Device Server not responding, the application will be terminated immediately" << endl;
		exit(1);
	}
	refresh();
	// init timer
	try {
		timer = new QTimer(this);
		timer->setInterval(1000);
		timer->setSingleShot(false);
		connect(timer, SIGNAL(timeout()), this, SLOT(refresh()));
		timer->start();
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "cannot connect UI" << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception, cannot setup UI" << endl;
	}
}

acstoroid::~acstoroid()
{
}    
    
int acstoroid::myReadBoolArray(const char *attrName, vector<bool> & attrVal)
{
	try {
		plc_server->read_attribute(attrName) >> attrVal;
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}
    
int acstoroid::myReadDouble(const char *attrName, double *attrVal)
{
	try {
		plc_server->read_attribute(attrName) >> *attrVal;
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << tmBuffer << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}

int acstoroid::myReadAttribute(const char *attrName)
{
	try {
		myAttribute = plc_server->read_attribute(attrName);
		return 1;
	}
	catch (const Tango::DevFailed &e) {
		timestamp();
		cout << "error reading attribute: " << attrName << endl;
	}
	catch (...) {
		timestamp();
		cout << tmBuffer << "default exception reading attribute: " << attrName << endl;
	}
	return 0;
}

void acstoroid::refresh()
{
	vector<bool> stat, stat2;
	vector<short> caus;
	double re;
	char res[80];

	if (myReadDouble("Undulator_cm_mbd_01", &re)) {
		sprintf(res, "%6.3f", re);
		ui.cm_mbd_01->setText(res);
	}	
	if (myReadDouble("Undulator_cm_mbd_fel_01_01", &re)) {
		sprintf(res, "%6.3f", re);
		ui.cm_mbd_fel_01_01->setText(res);
	}	
	if (myReadDouble("Undulator_cm_mbd_fel_02_01", &re)) {
		sprintf(res, "%6.3f", re);
		ui.cm_mbd_fel_02_01->setText(res);
	}	
	if (myReadDouble("Undulator_cm_sfel_01_01", &re)) {
		sprintf(res, "%6.3f", re);
		ui.cm_sfel_01_01->setText(res);
	}	
	if (myReadDouble("Undulator_cm_sfel_02_01", &re)) {
		sprintf(res, "%6.3f", re);
		ui.cm_sfel_02_01->setText(res);
	}	
}
